package mcmultipart.client.multipart;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.google.common.base.Predicate;

import mcmultipart.block.BlockCoverable;
import mcmultipart.block.BlockMultipartContainer;
import mcmultipart.multipart.PartState;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.block.model.BakedQuad;
import net.minecraft.client.renderer.block.model.IBakedModel;
import net.minecraft.client.renderer.block.model.ItemCameraTransforms;
import net.minecraft.client.renderer.block.model.ItemOverrideList;
import net.minecraft.client.renderer.block.model.ModelResourceLocation;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.util.BlockRenderLayer;
import net.minecraft.util.EnumFacing;
import net.minecraftforge.client.MinecraftForgeClient;
import net.minecraftforge.common.property.IExtendedBlockState;

@SuppressWarnings("deprecation")
public class ModelMultipartContainer implements IBakedModel {

    final IBakedModel model;
    private final Predicate<BlockRenderLayer> layerFilter;

    public ModelMultipartContainer(IBakedModel model, Predicate<BlockRenderLayer> layerFilter) {

        this.model = model;
        this.layerFilter = layerFilter;
    }

    @Override
    public List<BakedQuad> func_188616_a(IBlockState state, EnumFacing side, long rand) {

        if (state == null) {
            if (model != null) return model.func_188616_a(state, side, rand);
            return Collections.emptyList();
        }
        BlockRenderLayer layer = MinecraftForgeClient.getRenderLayer();
        if (!(state instanceof IExtendedBlockState) || !((IExtendedBlockState) state).getUnlistedProperties()
                .containsKey(BlockMultipartContainer.PROPERTY_MULTIPART_CONTAINER)) {
            if (model != null && layerFilter.apply(layer)) return model.func_188616_a(state, side, rand);
            return Collections.emptyList();
        }
        List<PartState> partStates = ((IExtendedBlockState) state).getValue(BlockMultipartContainer.PROPERTY_MULTIPART_CONTAINER);
        List<BakedQuad> quads = new ArrayList<BakedQuad>();

        if (model != null && layerFilter.apply(layer)) quads.addAll(model.func_188616_a(state, side, rand));

        for (PartState partState : partStates) {
            if (!partState.renderLayers.contains(MinecraftForgeClient.getRenderLayer())) continue;

            ModelResourceLocation modelLocation = new ModelResourceLocation(partState.modelPath,
                    MultipartStateMapper.instance.func_178131_a(partState.state.func_177228_b()));
            IBakedModel model = Minecraft.func_71410_x().func_175602_ab().func_175023_a().func_178126_b()
                    .func_174953_a(modelLocation);
            if (model != null) quads.addAll(model.func_188616_a(partState.state, side, rand));
        }

        return quads;
    }

    @Override
    public boolean func_177555_b() {

        return model != null ? model.func_177555_b() : true;
    }

    @Override
    public boolean func_177556_c() {

        return model != null ? model.func_177556_c() : false;
    }

    @Override
    public boolean func_188618_c() {

        return model != null ? model.func_188618_c() : false;
    }

    @Override
    public TextureAtlasSprite func_177554_e() {

        return model != null ? model.func_177554_e()
                : Minecraft.func_71410_x().func_147117_R().func_110572_b("minecraft:blocks/stone");
    }

    @Override
    public ItemCameraTransforms func_177552_f() {

        return model != null ? model.func_177552_f() : ItemCameraTransforms.field_178357_a;
    }

    @Override
    public ItemOverrideList func_188617_f() {

        return ItemOverrideList.field_188022_a;
    }

    public static ModelMultipartContainer fromBlock(IBakedModel model, final BlockCoverable block) {

        return new ModelMultipartContainer(model, new Predicate<BlockRenderLayer>() {

            @Override
            public boolean apply(BlockRenderLayer layer) {

                return block.canRenderInLayerDefault(layer);
            }
        });
    }

}
