package mcmultipart.client.multipart;

import mcmultipart.block.TileCoverable;
import mcmultipart.block.TileMultipartContainer;
import mcmultipart.multipart.IMultipart;
import mcmultipart.multipart.IMultipartContainer;
import mcmultipart.multipart.MultipartRegistry;
import mcmultipart.raytrace.PartMOP;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.block.model.BakedQuad;
import net.minecraft.client.renderer.block.model.IBakedModel;
import net.minecraft.client.renderer.block.model.ModelResourceLocation;
import net.minecraft.client.renderer.block.model.SimpleBakedModel;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.client.renderer.tileentity.TileEntityRendererDispatcher;
import net.minecraft.client.renderer.tileentity.TileEntitySpecialRenderer;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.util.BlockRenderLayer;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.RayTraceResult;
import net.minecraftforge.client.ForgeHooksClient;
import net.minecraftforge.client.MinecraftForgeClient;
import net.minecraftforge.client.model.pipeline.IVertexConsumer;
import net.minecraftforge.client.model.pipeline.VertexBufferConsumer;

public final class MultipartContainerSpecialRenderer {

    private static final EnumFacing[] ALL_ENUM_FACING = new EnumFacing[] { EnumFacing.DOWN, EnumFacing.UP, EnumFacing.NORTH,
            EnumFacing.SOUTH, EnumFacing.WEST, EnumFacing.EAST, null };

    public static boolean renderMultipartContainerAt(IMultipartContainer container, double x, double y, double z, float partialTicks,
            int destroyStage, TileEntityRendererDispatcher rendererDispatcher) {

        if (destroyStage >= 0) {
            IVertexConsumer consumer = new VertexBufferConsumer(Tessellator.func_178181_a().func_178180_c());
            startBreaking(rendererDispatcher);

            RayTraceResult mop = Minecraft.func_71410_x().field_71476_x;
            if (mop != null && mop.field_72313_a == RayTraceResult.Type.BLOCK && mop.func_178782_a() != null
                    && mop.func_178782_a().equals(container.getPosIn()) && !(mop instanceof PartMOP)) {
                renderBreaking(((PartMOP) mop).partHit, consumer, x, y, z, partialTicks, destroyStage, rendererDispatcher);
            } else {
                for (IMultipart part : container.getParts())
                    renderBreaking(part, consumer, x, y, z, partialTicks, destroyStage, rendererDispatcher);
            }

            finishBreaking();
            return true;
        }

        for (IMultipart part : container.getParts()) {
            MultipartSpecialRenderer<IMultipart> renderer = MultipartRegistryClient.getSpecialRenderer(part);
            if (renderer != null && renderer.shouldRenderInPass(part, MinecraftForgeClient.getRenderPass())) {
                renderer.setRendererDispatcher(rendererDispatcher);
                renderer.renderMultipartAt(part, x, y, z, partialTicks, destroyStage);
            }
        }
        return false;
    }

    private static void startBreaking(TileEntityRendererDispatcher rendererDispatcher) {

        GlStateManager.func_179094_E();
        GlStateManager.func_179120_a(774, 768, 1, 0);
        GlStateManager.func_179147_l();
        GlStateManager.func_179131_c(1.0F, 1.0F, 1.0F, 0.5F);
        GlStateManager.func_179136_a(-3.0F, -3.0F);
        GlStateManager.func_179088_q();
        GlStateManager.func_179092_a(516, 0.1F);
        GlStateManager.func_179141_d();
        Minecraft.func_71410_x().func_110434_K().func_110581_b(TextureMap.field_110575_b).func_174936_b(false, false);
    }

    private static void startTessellating(double x, double y, double z) {

        Tessellator.func_178181_a().func_178180_c().func_181668_a(7, DefaultVertexFormats.field_181703_c);
        Tessellator.func_178181_a().func_178180_c().func_178969_c(x, y, z);
        Tessellator.func_178181_a().func_178180_c().func_78914_f();
    }

    private static void renderBreaking(IMultipart part, IVertexConsumer consumer, double x, double y, double z, float partialTicks,
            int destroyStage, TileEntityRendererDispatcher rendererDispatcher) {

        MultipartSpecialRenderer<IMultipart> renderer = MultipartRegistryClient.getSpecialRenderer(part);
        if (renderer != null && renderer.canRenderBreaking(part)) {
            renderer.setRendererDispatcher(rendererDispatcher);
            renderer.renderMultipartAt(part, x, y, z, partialTicks, destroyStage);
        } else {
            if (MinecraftForgeClient.getRenderPass() == 1) {
                ResourceLocation path = part.getModelPath();
                IBlockState state = part.getExtendedState(MultipartRegistry.getDefaultState(part).func_177621_b());
                IBakedModel model = path == null ? null
                        : Minecraft.func_71410_x().func_175602_ab().func_175023_a().func_178126_b().func_174953_a(
                                new ModelResourceLocation(path, MultipartStateMapper.instance.func_178131_a(state.func_177228_b())));
                if (model != null) {
                    for (BlockRenderLayer layer : BlockRenderLayer.values()) {
                        if (part.canRenderInLayer(layer)) {
                            ForgeHooksClient.setRenderLayer(layer);
                            IBakedModel layerModel  = (new SimpleBakedModel.Builder(state, model,
                                    Minecraft.func_71410_x().func_147117_R()
                                            .func_110572_b("minecraft:blocks/destroy_stage_" + destroyStage),
                                    part.getPos())).func_177645_b();
                            rendererDispatcher.field_147553_e.func_110577_a(TextureMap.field_110575_b);
                            startTessellating(x, y, z);
                            consumer = new VertexBufferConsumer(Tessellator.func_178181_a().func_178180_c());
                            renderBreaking(state, layerModel, consumer);
                            finishTessellating();
                        }
                    }
                    ForgeHooksClient.setRenderLayer(BlockRenderLayer.SOLID);
                }
            }
        }
    }

    private static void renderBreaking(IBlockState state, IBakedModel model, IVertexConsumer consumer) {

        for (EnumFacing side : ALL_ENUM_FACING)
            for (BakedQuad quad : model.func_188616_a(state, side, 0L))
                quad.pipe(consumer);
    }

    private static void finishTessellating() {

        Tessellator.func_178181_a().func_178180_c().func_178969_c(0, 0, 0);
        Tessellator.func_178181_a().func_78381_a();
    }

    private static void finishBreaking() {

        Minecraft.func_71410_x().func_110434_K().func_110581_b(TextureMap.field_110575_b).func_174935_a();
        GlStateManager.func_179118_c();
        GlStateManager.func_179136_a(0.0F, 0.0F);
        GlStateManager.func_179113_r();
        GlStateManager.func_179141_d();
        GlStateManager.func_179132_a(true);
        GlStateManager.func_179121_F();
    }

    public static final class TileMultipartSpecialRenderer extends TileEntitySpecialRenderer<TileMultipartContainer> {

        @Override
        public void func_180535_a(TileMultipartContainer te, double x, double y, double z, float partialTicks, int destroyStage) {

            renderMultipartContainerAt(te, x, y, z, partialTicks, destroyStage, field_147501_a);
        }

    }

    public static class TileCoverableSpecialRenderer<T extends TileCoverable> extends TileEntitySpecialRenderer<T> {

        @Override
        public void func_180535_a(T te, double x, double y, double z, float partialTicks, int destroyStage) {

            if (destroyStage >= 0) {
                if (MinecraftForgeClient.getRenderPass() != 1) return;

                RayTraceResult mop = Minecraft.func_71410_x().field_71476_x;
                if (mop != null && mop.field_72313_a == RayTraceResult.Type.BLOCK && mop.func_178782_a() != null
                        && mop.func_178782_a().equals(te.getPosIn()) && !(mop instanceof PartMOP)) {
                    IVertexConsumer consumer = new VertexBufferConsumer(Tessellator.func_178181_a().func_178180_c());
                    startBreaking(field_147501_a);
                    if (canRenderBreaking()) {
                        renderTileEntityAtDefault(te, x, y, z, partialTicks, destroyStage);
                    } else {
                        IBlockState state = te.getWorldIn().func_180495_p(te.getPosIn());
                        IBakedModel model = Minecraft.func_71410_x().func_175602_ab().func_175023_a()
                                .func_178125_b(te.func_145838_q().func_176221_a(state, te.getWorldIn(), te.getPosIn()));
                        if (model != null && model instanceof ModelMultipartContainer) model = ((ModelMultipartContainer) model).model;
                        if (model != null) {
                            model = (new SimpleBakedModel.Builder(state, model,
                                    Minecraft.func_71410_x().func_147117_R()
                                            .func_110572_b("minecraft:blocks/destroy_stage_" + destroyStage),
                                    te.getPosIn())).func_177645_b();
                            startTessellating(x, y, z);
                            renderBreaking(state, model, consumer);
                            finishTessellating();
                        }
                    }

                    finishBreaking();
                    return;
                }
            }

            if (renderMultipartContainerAt(te.getMicroblockContainer(), x, y, z, partialTicks, destroyStage, field_147501_a)) return;
            renderTileEntityAtDefault(te, x, y, z, partialTicks, destroyStage);
        }

        public void renderTileEntityAtDefault(T te, double x, double y, double z, float partialTicks, int destroyStage) {

        }

        public boolean canRenderBreaking() {

            return false;
        }

    }

}
